/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@users.sf.net>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2001-2004 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.ui;

import java.io.InputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Map;
import org.w3c.dom.Document;
import com.silverwrist.util.ServletMultipartException;
import com.silverwrist.venice.core.CommunityContext;
import com.silverwrist.venice.core.UserContext;
import com.silverwrist.venice.core.VeniceEngine;
import com.silverwrist.venice.except.*;
import com.silverwrist.venice.ui.dlg.Dialog;
import com.silverwrist.venice.ui.helpers.ErrorBox;
import com.silverwrist.venice.ui.menus.MenuComponent;
import com.silverwrist.venice.ui.script.ScriptManager;
import com.silverwrist.venice.util.ServiceProvider;

public interface RequestInput extends ServiceProvider
{
  public static final String LEFT_MENU_SESSION_ATTR = "LeftMenu";

  /**
   * Returns a {@link java.lang.String String} containing the real path for a given virtual path. For example,
   * the virtual path &quot;/index.html&quot; has a real path of whatever file on the server's filesystem
   * would be served by a request for &quot;/index.html&quot;.<p>
   * This method returns <code>null</code> if the virtual path cannot be translated to a real path for
   * any reason.
   *
   * @param s A <code>String</code> specifying a virtual path.
   * @return A <code>String</code> specifying the real path, or <code>null</code> if the translation cannot
   *         be performed.
   */
  public String mapPath(String s);

  /**
   * Returns the portion of the request URI that indicates the context of the request, i.e. the servlet
   * context under which Venice is installed. The context path always comes first in a request URI. The
   * path starts with a &quot;/&quot; character but does not end with a &quot;/&quot; character.
   *
   * @return A {@link java.lang.String String} specifying the portion of the request URI that indicates the
   *         context of the request.
   */
  public String getContextPath();

  /**
   * Returns the part of this request's URI that calls the servlet. This includes either the servlet name
   * or a path to the servlet, but does not include any extra path information or a query string.
   *
   * @return A {@link java.lang.String String} containing the name or path of the servlet being called, as
   *         specified in the request URI.
   */
  public String getServletPath();

  /**
   * Returns any extra path information associated with the URI the client sent when it made this request.
   * The extra path information follows the servlet path but precedes the query string. This method
   * returns <code>null</code> if there was no extra path information.  In Venice, the path information is
   * used to specify parameters in certain servlets.
   *
   * @return A {@link java.lang.String String} specifying extra path information that comes after the servlet path
   *         but before the query string in the request URI; or <code>null</code> if the URI does not
   *         have any extra path information.
   */
  public String getPathInfo();

  /**
   * Returns the query string that is contained in the request URI after the path. This method returns
   * <code>null</code> if the URI does not have a query string.
   *
   * @return A {@link java.lang.String String} containing the query string or <code>null</code> if the URI contains
   *         no query string.
   */
  public String getQueryString();

  /**
   * Returns the name of the HTTP method with which this request was made, for example, GET, POST, or PUT.
   *
   * @return A {@link java.lang.String String} specifying the name of the method with which this request was made.
   */
  public String getVerb();

  /**
   * Returns the Internet Protocol (IP) address of the client that sent the request.
   *
   * @return A {@link java.lang.String String} containing the IP address of the client that sent the request.
   */
  public String getSourceAddress();

  /**
   * Returns <code>true</code> if the request parameter with the specified name is defined,
   * <code>false</code> if not.  For standard Venice requests, parameters are contained in the
   * query string or posted form data.
   *
   * @param name Parameter name to be checked.
   * @return See above.
   */
  public boolean hasParameter(String name);

  /**
   * Returns the value of a request parameter as a {@link java.lang.String String}, or <code>null</code> if the
   * parameter does not exist. For standard Venice requests, parameters are contained in the query
   * string or posted form data.<p>
   * You should only use this method when you are sure the parameter has only one value. If the parameter
   * might have more than one value, use {@link #getParameterValues(java.lang.String)}.<p>
   * If you use this method with a multivalued parameter, the value returned is equal to the first value
   * in the array returned by <code>getParameterValues</code>.
   *
   * @param name A <code>String</code> specifying the name of the parameter.
   * @return A <code>String</code> representing the single value of the parameter.
   * @see #getParameterValues(java.lang.String)
   */
  public String getParameter(String name);

  /**
   * Returns the value of a request parameter as a <code>int</code>, or a specified default value if
   * the parameter does not exist. For standard Venice requests, parameters are contained in the query
   * string or posted form data.<p>
   * If you use this method with a multivalued parameter, the value returned is equal to the
   * <code>int</code> equivalent of the first value in the array returned by
   * {@link #getParameterValues(java.lang.String)}.
   *
   * @param name A {@link java.lang.String String} specifying the name of the parameter.
   * @param default_value The default value to use for the parameter if the specified parameter does not exist.
   * @return An <code>int</code> representing the single value of the parameter.
   */
  public int getParameterInt(String name, int default_value);

  /**
   * Returns the value of a request parameter as a <code>short</code>, or a specified default value if
   * the parameter does not exist. For standard Venice requests, parameters are contained in the query
   * string or posted form data.<p>
   * If you use this method with a multivalued parameter, the value returned is equal to the
   * <code>short</code> equivalent of the first value in the array returned by
   * {@link #getParameterValues(java.lang.String)}.
   *
   * @param name A {@link java.lang.String String} specifying the name of the parameter.
   * @param default_value The default value to use for the parameter if the specified parameter does not exist.
   * @return A <code>short</code> representing the single value of the parameter.
   */
  public short getParameterShort(String name, short default_value);

  /**
   * Returns the value of a request parameter as a <code>long</code>, or a specified default value if
   * the parameter does not exist. For standard Venice requests, parameters are contained in the query
   * string or posted form data.<p>
   * If you use this method with a multivalued parameter, the value returned is equal to the
   * <code>long</code> equivalent of the first value in the array returned by
   * {@link #getParameterValues(java.lang.String)}.
   *
   * @param name A {@link java.lang.String String} specifying the name of the parameter.
   * @param default_value The default value to use for the parameter if the specified parameter does not exist.
   * @return A <code>long</code> representing the single value of the parameter.
   */
  public long getParameterLong(String name, long default_value);

  /**
   * Returns an {@link java.util.Enumeration Enumeration} of {@link java.lang.String String} objects containing
   * the names of the parameters contained in this request. If the request has no parameters, the method returns
   * an empty <code>Enumeration.</code>
   *
   * @return An <code>Enumeration</code> of <code>String</code> objects, each <code>String</code>
   *         containing the name of a request parameter, or an empty <code>Enumeration</code> if the
   *         request has no parameters.
   */
  public Enumeration getParameterNames();

  /**
   * Returns an array of {@link java.lang.String String} objects containing all of the values the given request
   * parameter has, or <code>null</code> if the parameter does not exist.<p>
   * If the parameter has a single value, the array has a length of 1.
   *
   * @param name A <code>String</code> containing the name of the parameter whose value is requested.
   * @return An array of <code>String</code> objects containing the parameter's values.
   * @see #getParameter(java.lang.String)
   */
  public String[] getParameterValues(String name);

  /**
   * Returns <code>true</code> if the specified parameter is a file parameter, <code>false</code> if
   * not.  If the parameter does not exist, the return value is <code>false</code>.
   *
   * @param name A {@link java.lang.String String} containing the name of the parameter to test.
   * @return See above.
   */
  public boolean isFileParam(String name);

  /**
   * Returns the MIME type of the specified parameter.  If the type cannot be determined, the return
   * value is <code>null</code>.<p>
   * N.B.: For non-file parameters, or all parameters to a request that is not a file upload, the MIME
   * type is &quot;text/plain&quot;.
   *
   * @param name A {@link java.lang.String String} containing the name of the parameter to test.
   * @return See above.
   */
  public String getParameterType(String name);

  /**
   * Returns the size in bytes of the specified parameter.  If the size cannot be determined, the return
   * value is -1.<p>
   * N.B.: For non-file parameters, or all parameters to a request that is not a file upload, the size
   * is the number of bytes occupied by the parameter, expressed in UTF-8 encoding.
   *
   * @param name A {@link java.lang.String String} containing the name of the parameter to test.
   * @return See above.
   */
  public int getParameterSize(String name);

  /**
   * Returns an {@link java.io.InputStream InputStream} reading from the data of the named file parameter.  If the
   * named parameter does not exist or is not a file parameter, returns <code>null;
   *
   * @param name A {@link java.lang.String String} containing the name of the parameter whose value is requested.
   * @return See above.
   * @exception com.silverwrist.util.ServletMultipartException If there is a problem retrieving
   *            the parameter data stream.
   */
  public InputStream getParameterDataStream(String name) throws ServletMultipartException;

  /**
   * Returns <code>true</code> if the parameter set reflects the clicking of an image button with a
   * specified name, <code>false</code> if not.
   *
   * @param name A {@link java.lang.String String} containing the name of the image button to test.
   * @return See above.
   */
  public boolean isImageButtonClicked(String name);

  /**
   * Returns the application-level attribute with the given name, or <code>null</code> if there is no
   * attribute by that name.
   *
   * @param name A {@link java.lang.String String} specifying the name of the attribute.
   * @return An {@link java.lang.Object Object} containing the value of the attribute, or <code>null</code> if no
   *         attribute exists matching the given name.
   */
  public Object getAppAttribute(String name);

  /**
   * Sets the application-level attribute with the given name.
   *
   * @param name A {@link java.lang.String String} specifying the name of the attribute.
   * @param o The object to be bound as the value of that attribute, or <code>null</code> if the binding
   *          is to be removed.
   */
  public void setAppAttribute(String name, Object o);

  /**
   * Returns the session-level attribute with the given name, or <code>null</code> if there is no
   * attribute by that name.
   *
   * @param name A {@link java.lang.String String} specifying the name of the attribute.
   * @return An {@link java.lang.Object Object} containing the value of the attribute, or <code>null</code> if no
   *         attribute exists matching the given name.
   */
  public Object getSessionAttribute(String name);

  /**
   * Sets the session-level attribute with the given name.
   *
   * @param name A {@link java.lang.String String} specifying the name of the attribute.
   * @param o The object to be bound as the value of that attribute, or <code>null</code> if the binding
   *          is to be removed.
   */
  public void setSessionAttribute(String name, Object o);

  /**
   * Returns the request-level attribute with the given name, or <code>null</code> if there is no
   * attribute by that name.
   *
   * @param name A {@link java.lang.String String} specifying the name of the attribute.
   * @return An {@link java.lang.Object Object} containing the value of the attribute, or <code>null</code> if no
   *         attribute exists matching the given name.
   */
  public Object getRequestAttribute(String name);

  /**
   * Sets the request-level attribute with the given name.
   *
   * @param name A {@link java.lang.String String} specifying the name of the attribute.
   * @param o The object to be bound as the value of that attribute, or <code>null</code> if the binding
   *          is to be removed.
   */
  public void setRequestAttribute(String name, Object o);

  /**
   * Returns the instance of the Venice engine associated with the application.
   *
   * @return See above.
   */
  public VeniceEngine getEngine();

  /**
   * Returns the instance of the Venice user object associated with the session.
   *
   * @return See above.
   */
  public UserContext getUser();

  /**
   * Returns the current servlet location.  This is used, for instance, as the context to return to
   * after a login.
   *
   * @return The current servlet location.
   */
  public String getLocation();

  /**
   * Sets the &quot;current&quot; servlet location that is displayed.  This is used, for instance,
   * as the context to return to after a login.
   *
   * @param str The new location to be set.
   */
  public void setLocation(String str);

  /**
   * Returns <code>true</code> if the &quot;Log In&quot; link is to be displayed on the outer frame,
   * <code>false</code> if not.
   *
   * @return See above.
   */
  public boolean getDisplayLogin();

  /**
   * Sets whether or not the &quot;Log In&quot; link is to be displayed on the outer frame.
   *
   * @param val <code>true</code> to display the &quot;Log In&quot; link on the outer frame,
   *            <code>false</code> to omit it.
   */
  public void setDisplayLogin(boolean val);

  public MenuComponent getMenu(String name);

  public MenuComponent getMenu(String name, Map vars);

  public Dialog getDialog(String name);

  public Content[] getSideBoxes() throws AccessError, DataException;

  public CommunityContext getCommunity();

  public CommunityContext getCommunity(boolean required, String on_error) throws ErrorBox;

  public String getDefaultServletAddress(CommunityContext comm);

  public void registerCleanup(AutoCleanup ac);

  public String getConfigProperty(String name);

  public String getConfigProperty(String name, String default_val);

  public void setSpecial(String name, String value);

} // end interface RequestInput
